// -*-c++-*-

/***************************************************************************
                          rccaction.h  -  Interface and Implementation of
			                  RoboCup coach language actions
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_ACTION_H_
#define _RCC_ACTION_H_

#include <iostream>
#include <vector>

#include "rccobject.h"
#include "rccregion.h"
#include "rccmemory.h"
#include "rccstring.h"
#include "rccexception.h"

class RCC_Action : public RCC_Object
{
public:
  RCC_Action() {}
  virtual ~RCC_Action() {}

  virtual std::ostream& print ( std::ostream& out ) const = 0;
  virtual std::ostream& printPretty ( std::ostream& out, const RCC_String& line_header ) const = 0;
  friend std::ostream& operator << ( std::ostream & os, RCC_Action& a )
    { return a.print(os); } 
  
};


class RCC_ActPosition;
class RCC_ActHome;
class RCC_ActBallToRegion;
class RCC_ActBallToPlayer;
class RCC_ActMark;
class RCC_ActMarkLinePlayer;
class RCC_ActMarkLineRegion;
class RCC_ActOffsidesLine;
class RCC_ActHetType;
class RCC_ActNamed;

class RCC_ActUnion
  : public RCC_Object
{
public:
  enum type_t
  {
    ACT_VOID,
    ACT_POS,
    ACT_HOME,
    ACT_BALLR,
    ACT_BALLP,
    ACT_MARK,
    ACT_MARKP,
    ACT_MARKR,
    ACT_OFF,
    ACT_HET,
    ACT_NAMED
  };

  RCC_ActUnion ( RCC_Pointer < RCC_ActPosition > act )
    :
    act_p ( act ),
    act_pos_p ( act ),
    act_type ( RCC_ActUnion::ACT_POS )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActHome > act )
    :
    act_p ( act ),
    act_home_p ( act ),
    act_type ( RCC_ActUnion::ACT_HOME )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActBallToRegion > act )
    :
    act_p ( act ),
    act_ballr_p ( act ),
    act_type ( RCC_ActUnion::ACT_BALLR )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActBallToPlayer > act )
    :
    act_p ( act ),
    act_ballp_p ( act ),
    act_type ( RCC_ActUnion::ACT_BALLP )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActMark > act )
    :
    act_p ( act ),
    act_mark_p ( act ),
    act_type ( RCC_ActUnion::ACT_MARK )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActMarkLinePlayer > act )
    :
    act_p ( act ),
    act_markp_p ( act ),
    act_type ( RCC_ActUnion::ACT_MARKP )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActMarkLineRegion > act )
    :
    act_p ( act ),
    act_markr_p ( act ),
    act_type ( RCC_ActUnion::ACT_MARKR )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActOffsidesLine > act )
    :
    act_p ( act ),
    act_off_p ( act ),
    act_type ( RCC_ActUnion::ACT_OFF )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActHetType > act )
    :
    act_p ( act ),
    act_het_p ( act ),
    act_type ( RCC_ActUnion::ACT_HET )
  {}

  RCC_ActUnion ( RCC_Pointer < RCC_ActNamed > act )
    :
    act_p ( act ),
    act_named_p ( act ),
    act_type ( RCC_ActUnion::ACT_NAMED )
  {}

  ~RCC_ActUnion () {}

  RCC_Pointer < RCC_Object >
  copy () const
  {
    switch ( act_type )
      {
      case RCC_ActUnion::ACT_POS:
	{
	  RCC_Pointer < RCC_ActPosition > act_copy =
	    act_p->copy ().staticCast < RCC_ActPosition > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_HOME:
	{
	  RCC_Pointer < RCC_ActHome > act_copy =
	    act_p->copy ().staticCast < RCC_ActHome > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_BALLR:
	{
	  RCC_Pointer < RCC_ActBallToRegion > act_copy =
	    act_p->copy ().staticCast < RCC_ActBallToRegion > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_BALLP:
	{
	  RCC_Pointer < RCC_ActBallToPlayer > act_copy =
	    act_p->copy ().staticCast < RCC_ActBallToPlayer > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_MARK:
	{
	  RCC_Pointer < RCC_ActMark > act_copy =
	    act_p->copy ().staticCast < RCC_ActMark > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_MARKP:
	{
	  RCC_Pointer < RCC_ActMarkLinePlayer > act_copy =
	    act_p->copy ().staticCast < RCC_ActMarkLinePlayer > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_MARKR:
	{
	  RCC_Pointer < RCC_ActMarkLineRegion > act_copy =
	    act_p->copy ().staticCast < RCC_ActMarkLineRegion > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_OFF:
	{
	  RCC_Pointer < RCC_ActOffsidesLine > act_copy =
	    act_p->copy ().staticCast < RCC_ActOffsidesLine > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_HET:
	{
	  RCC_Pointer < RCC_ActHetType > act_copy =
	    act_p->copy ().staticCast < RCC_ActHetType > ();
	  return new RCC_ActUnion ( act_copy );
	}
      case RCC_ActUnion::ACT_NAMED:
	{
	  RCC_Pointer < RCC_ActNamed > act_copy =
	    act_p->copy ().staticCast < RCC_ActNamed > ();
	  return new RCC_ActUnion ( act_copy );
	}
      default:
	return NULL;
      };
  }

  RCC_ActUnion::type_t
  type () const
  { return act_type; }

  RCC_Pointer < RCC_ActPosition >
  setAct ( RCC_Pointer < RCC_ActPosition > act )
  {
    act_p = act;
    act_pos_p = act;
    act_type = RCC_ActUnion::ACT_POS;
  }

  RCC_Pointer < RCC_ActHome >
  setAct ( RCC_Pointer < RCC_ActHome > act )
  {
    act_p = act;
    act_home_p = act;
    act_type = RCC_ActUnion::ACT_POS;
  }

  RCC_Pointer < RCC_ActBallToRegion >
  setAct ( RCC_Pointer < RCC_ActBallToRegion > act )
  {
    act_p = act;
    act_ballr_p = act;
    act_type = RCC_ActUnion::ACT_BALLR;
  }

  RCC_Pointer < RCC_ActBallToPlayer >
  setAct ( RCC_Pointer < RCC_ActBallToPlayer > act )
  {
    act_p = act;
    act_ballp_p = act;
    act_type = RCC_ActUnion::ACT_BALLP;
  }

  RCC_Pointer < RCC_ActMark >
  setAct ( RCC_Pointer < RCC_ActMark > act )
  {
    act_p = act;
    act_mark_p = act;
    act_type = RCC_ActUnion::ACT_MARK;
  }

  RCC_Pointer < RCC_ActMarkLinePlayer >
  setAct ( RCC_Pointer < RCC_ActMarkLinePlayer > act )
  {
    act_p = act;
    act_markp_p = act;
    act_type = RCC_ActUnion::ACT_MARKP;
  }

  RCC_Pointer < RCC_ActMarkLineRegion >
  setAct ( RCC_Pointer < RCC_ActMarkLineRegion > act )
  {
    act_p = act;
    act_markr_p = act;
    act_type = RCC_ActUnion::ACT_MARKR;
  }

  RCC_Pointer < RCC_ActOffsidesLine >
  setAct ( RCC_Pointer < RCC_ActOffsidesLine > act )
  {
    act_p = act;
    act_off_p = act;
    act_type = RCC_ActUnion::ACT_OFF;
  }

  RCC_Pointer < RCC_ActHetType >
  setAct ( RCC_Pointer < RCC_ActHetType > act )
  {
    act_p = act;
    act_het_p = act;
    act_type = RCC_ActUnion::ACT_HET;
  }

  RCC_Pointer < RCC_ActNamed >
  setAct ( RCC_Pointer < RCC_ActNamed > act )
  {
    act_p = act;
    act_named_p = act;
    act_type = RCC_ActUnion::ACT_NAMED;
  }


  RCC_Pointer < RCC_ActPosition >
  actPos ()
  { 
    if ( act_type == RCC_ActUnion::ACT_POS )
      return act_pos_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActPosition >
  actPos () const
  { 
    if ( act_type == RCC_ActUnion::ACT_POS )
      return act_pos_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActHome >
  actHome ()
  { 
    if ( act_type == RCC_ActUnion::ACT_HOME )
      return act_home_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActHome >
  actHome () const
  { 
    if ( act_type == RCC_ActUnion::ACT_HOME )
      return act_home_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActBallToRegion >
  actBallR ()
  { 
    if ( act_type == RCC_ActUnion::ACT_BALLR )
      return act_ballr_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActBallToRegion >
  actBallR () const
  { 
    if ( act_type == RCC_ActUnion::ACT_BALLR )
      return act_ballr_p;
    return NULL;
  }
 
  RCC_Pointer < RCC_ActBallToPlayer >
  actBallP ()
  { 
    if ( act_type == RCC_ActUnion::ACT_BALLP )
      return act_ballp_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActBallToPlayer >
  actBallP () const
  { 
    if ( act_type == RCC_ActUnion::ACT_BALLP )
      return act_ballp_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActMark >
  actMark ()
  { 
    if ( act_type == RCC_ActUnion::ACT_MARK )
      return act_mark_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActMark >
  actMark () const
  { 
    if ( act_type == RCC_ActUnion::ACT_MARK )
      return act_mark_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActMarkLinePlayer >
  actMarkP ()
  { 
    if ( act_type == RCC_ActUnion::ACT_MARKP )
      return act_markp_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActMarkLinePlayer >
  actMarkP () const
  { 
    if ( act_type == RCC_ActUnion::ACT_MARKP )
      return act_markp_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActMarkLineRegion >
  actMarkR ()
  { 
    if ( act_type == RCC_ActUnion::ACT_MARKR )
      return act_markr_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActMarkLineRegion >
  actMarkR () const
  { 
    if ( act_type == RCC_ActUnion::ACT_MARKR )
      return act_markr_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActOffsidesLine >
  actOff ()
  { 
    if ( act_type == RCC_ActUnion::ACT_OFF )
      return act_off_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActOffsidesLine >
  actOff () const
  { 
    if ( act_type == RCC_ActUnion::ACT_OFF )
      return act_off_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActHetType >
  actHet ()
  { 
    if ( act_type == RCC_ActUnion::ACT_HET )
      return act_het_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActHetType >
  actHet () const
  { 
    if ( act_type == RCC_ActUnion::ACT_HET )
      return act_het_p;
    return NULL;
  }

  RCC_Pointer < RCC_ActNamed >
  actNamed ()
  { 
    if ( act_type == RCC_ActUnion::ACT_NAMED )
      return act_named_p;
    return NULL;
  }

  const RCC_Pointer < RCC_ActNamed >
  actNamed () const
  { 
    if ( act_type == RCC_ActUnion::ACT_NAMED )
      return act_named_p;
    return NULL;
  }

  RCC_Pointer < RCC_Action >
  act ()
  { 
    return act_p;
  }

  const RCC_Pointer < RCC_Action >
  act () const
  { 
    return act_p;
  }


private:
  RCC_Pointer < RCC_Action > act_p;
  union
  {
    RCC_ActPosition*		act_pos_p;
    RCC_ActHome*		act_home_p;
    RCC_ActBallToRegion*	act_ballr_p;
    RCC_ActBallToPlayer*	act_ballp_p;
    RCC_ActMark*		act_mark_p;
    RCC_ActMarkLinePlayer*	act_markp_p;
    RCC_ActMarkLineRegion*	act_markr_p;
    RCC_ActOffsidesLine*	act_off_p;
    RCC_ActHetType*		act_het_p;
    RCC_ActNamed*		act_named_p;
  };
  RCC_ActUnion::type_t	act_type;
};



class RCC_ActRegion 
  : public RCC_Action
{
public:
  RCC_ActRegion ( const RCC_Pointer < RCC_RegUnion > r ) 
    : reg_p ( r ) {} 
  ~RCC_ActRegion () {}
 
  /* class specific stuff */
  RCC_Pointer < RCC_RegUnion > 
  region () 
  { return reg_p; }
  
  const RCC_Pointer < RCC_RegUnion >
  region () const 
  { return reg_p; }
  
  RCC_Pointer < RCC_RegUnion >
  region ( const RCC_Pointer < RCC_RegUnion >& r ) 
  { return reg_p = r; }

private:
  RCC_Pointer < RCC_RegUnion > reg_p;
};


class RCC_ActPosition 
  : public RCC_ActRegion
{
public:
  RCC_ActPosition ( const RCC_Pointer < RCC_RegUnion >& r ) 
    : RCC_ActRegion ( r )  {}
  ~RCC_ActPosition() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_ActPosition ( reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(pos " << *( region ()->reg () ) << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { out << line_header << "position self at: " << std::endl;
    region ()->reg ()->printPretty ( out, line_header + " " ); 
    return out; }
};



class RCC_ActHome 
  : public RCC_ActRegion
{
public:
  RCC_ActHome ( const RCC_Pointer < RCC_RegUnion >& r ) : RCC_ActRegion ( r ) {}
  ~RCC_ActHome() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_ActHome ( reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(home " << *( region ()->reg () ) << ")"; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { out << line_header << "home position at: " << std::endl;
    region ()->reg ()->printPretty(out, line_header + " "); 
    return out; }
};


class RCC_ActBallToRegion
 : public RCC_ActRegion
{
public:
  RCC_ActBallToRegion ( const RCC_Pointer < RCC_RegUnion >& r, const RCC_BallMoveSet& bmset ) : RCC_ActRegion ( r ), bm_set ( bmset ) {}
  ~RCC_ActBallToRegion() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_ActBallToRegion ( reg_copy, bm_set ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "(bto " << *( region ()->reg () ) << " " << bm_set << ")"; }
  
  std::ostream& printPretty ( std::ostream& out, 
			      const RCC_String& line_header ) const
  { out << line_header << "ball to: use " << bm_set 
	<< " to go to:" << std::endl;
    region ()->reg ()->printPretty ( out, line_header + " " ); 
    return out; }

  /* class specific stuff */
  const RCC_BallMoveSet& BMSet() const { return bm_set; }
  void BMSet ( const RCC_BallMoveSet& bmset ) { bm_set = bmset; }
  void addBMT ( const RCC_BallMoveSet::RCC_BallMove_t& t ) { bm_set.add ( t ); }
  void removeBMT ( const RCC_BallMoveSet::RCC_BallMove_t& t ) { bm_set.remove ( t ); }
  void clearBMT() { bm_set.clear (); }


private:
  RCC_BallMoveSet bm_set;
} ;


class RCC_ActBallToPlayer
  : public RCC_Action
{
public:
  RCC_ActBallToPlayer ( const RCC_UnumSet& p ) : players_ ( p ) {}
  ~RCC_ActBallToPlayer() {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ActBallToPlayer ( *this ); }


  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "(bto " << players_ << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "ball to players: " 
	       << players_ << std::endl; }

  /* class specific stuff */
  const RCC_UnumSet& players() const { return players_; }
  void players ( const RCC_UnumSet& players ) { players_ = players; }
  void add ( const int& i ) { players_.add ( i ); }
  void remove ( const int& i ) { players_.remove ( i ); }
  void clear () { players_.clear(); }


private:
  RCC_UnumSet players_;
} ;


class RCC_ActMark
  : public RCC_Action
{
public:
  RCC_ActMark ( const unsigned int& u ) : unum ( u ) 
  {
    if ( unum == 0 || unum > MAX_PLAYER )
      throw RCC_BoundsException ( "Uniform number out of bounds" ); 
  }

  ~RCC_ActMark() {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ActMark ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(mark " << unum << ")"; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "mark opponent " << unum << std::endl; }

  /* class specific stuff */
  unsigned int 
  uNum () const 
  { return unum; }
  
  unsigned int
  uNum ( const unsigned int& u ) 
  { 
    if ( u == 0 || u > MAX_PLAYER ) 
      throw RCC_BoundsException ( "Uniform number out of bounds" );
    else
      return unum = u;
  } 


private:
  int unum;
};


class RCC_ActMarkLinePlayer 
  : public RCC_ActMark
{
public:
  RCC_ActMarkLinePlayer( const unsigned int& u ) : RCC_ActMark( u ) {}
  ~RCC_ActMarkLinePlayer() {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ActMarkLinePlayer ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(markl " << uNum () << ")"; }

  std::ostream& printPretty ( std::ostream& out, 
			      const RCC_String& line_header ) const
  { return out << line_header << "mark line to opponent " 
	       << uNum () << std::endl; }


};


class RCC_ActMarkLineRegion 
  : public RCC_ActRegion
{
public:
  RCC_ActMarkLineRegion ( const RCC_Pointer < RCC_RegUnion >& r ) : RCC_ActRegion ( r ) {}
  ~RCC_ActMarkLineRegion() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_ActMarkLineRegion ( reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(markl " << *( region ()->reg () ) << ")"; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { out << line_header << "mark line to region:" << std::endl;
  region ()->reg ()->printPretty ( out, line_header + " " ); 
  return out; }


} ;

class RCC_ActOffsidesLine 
  : public RCC_ActRegion
{
public:
  RCC_ActOffsidesLine ( const RCC_Pointer < RCC_RegUnion >& r ) : RCC_ActRegion ( r ) {}
  ~RCC_ActOffsidesLine() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_ActOffsidesLine ( reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(oline " << *( region ()->reg () ) << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { out << line_header << "offsides line at: " << std::endl;
  region ()->reg ()->printPretty ( out, line_header + " ");
  return out; }


} ;

class RCC_ActHetType 
  : public RCC_Action
{
public:
  RCC_ActHetType ( const unsigned int& pt ) : player_type ( pt ) {}
  ~RCC_ActHetType() {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ActHetType ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(htype " << player_type << ")"; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "heterogenous type: " 
	       << player_type << std::endl; }

  /* class specific stuff */
  unsigned int playerType () const { return player_type; }
  void playerType ( const unsigned int& pt ) { player_type = pt; }


private:
  int player_type;
};

class RCC_ActNamed 
  : public RCC_Action
{
public:
  RCC_ActNamed ( const RCC_String& n ) : name_ ( n ) {}
  ~RCC_ActNamed() {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ActNamed ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "\"" << name_ << "\"";  }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "action named \"" 
	       << name_ << "\"" << std::endl; }

  /* class specific */
  RCC_String name () const { return name_; }
  void  name ( const RCC_String& n ) { name_ = n; }


private:
  RCC_String name_;
};


#endif
