// -*-c++-*-

/***************************************************************************
                          rccaudio.h  -  Interface for RoboCup Client audio
			                 classes.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from 
                           Soccer Server's coach_lang_comp.h by the RoboCup
			   Soccer Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_AUDIO_H_
#define _RCC_AUDIO_H_

#pragma interface

#include "rccplaymode.h"
#include <iostream>
#include "rccangle.h"
#include "rcccoachmsg.h"
#include "rccside.h"
#include "rccsensordata.h"
#include "rccmemory.h"
#include "rccstring.h"

class RCC_Audio : public RCC_SensorData
{
public:
  RCC_Audio ( const int& t ) : time_ ( t ) {}
  virtual ~RCC_Audio ();
    
  virtual std::ostream& print ( std::ostream& o ) const = 0;

  friend std::ostream& operator<< ( std::ostream& o, const RCC_Audio& a ) { return a.print ( o ); }

  int time () const { return time_; }
  int time ( const int& t ) { return time_ = t; }
private:
  int time_;
};


class RCC_RefAudio : public RCC_Audio
{
public:
  RCC_RefAudio ( const int& t, const RCC_StadPlayMode& pm ) 
    : RCC_Audio ( t ), play_mode ( pm ) {}

  virtual ~RCC_RefAudio ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_RefAudio ( *this ); }

  virtual std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 


  RCC_StadPlayMode playMode () const { return play_mode; }
  RCC_StadPlayMode playMode ( const RCC_StadPlayMode& pm ) { return play_mode = pm; }
  
private:    
  RCC_StadPlayMode play_mode;
};


class RCC_SelfAudio : public RCC_Audio
{
public:
  RCC_SelfAudio ( const int& t, const RCC_String& m ) : RCC_Audio ( t ), msg_ ( m ) {}
  virtual ~RCC_SelfAudio ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_SelfAudio ( *this ); }

  virtual std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 


  RCC_String msg () const { return msg_; }
  RCC_String msg ( const RCC_String& m ) { return msg_ = m; }

private:   
  RCC_String msg_;
};


class RCC_PlayerAudio : public RCC_SelfAudio
{
public:
  RCC_PlayerAudio ( const int& t, const RCC_Angle& d, const RCC_String& m ) : RCC_SelfAudio ( t, m ), dir_ ( d ) {}
  virtual ~RCC_PlayerAudio ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_PlayerAudio ( *this ); }

  virtual std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 


  RCC_Angle dir () const { return dir_; }
  RCC_Angle dir ( const RCC_Angle& d ) { return dir_ = d; }
  
private:   
  RCC_Angle dir_;
};


class RCC_CoachAudio : public RCC_Audio
{
public:
  RCC_CoachAudio ( const int& t, 
		   const RCC_Side& s, 
		   const int &ts, 
		   RCC_Pointer < RCC_CoachMetaMsg > m ) 
    : 
    RCC_Audio ( t ), 
    side_ ( s ), 
    time_sent ( ts ), 
    msg_p ( ( RCC_CoachStdMsg* ) m ), 
    meta_p ( m ), 
    msg_type ( RCC_CoachStdMsg::CM_META ) 
  {}

  RCC_CoachAudio ( const int& t, 
		   const RCC_Side& s, 
		   const int &ts, 
		   RCC_Pointer < RCC_CoachFreeformMsg > m ) 
    : 
    RCC_Audio ( t ), 
    side_ ( s ), 
    time_sent ( ts ), 
    msg_p ( ( RCC_CoachStdMsg* ) m ), 
    free_p ( m ), 
    msg_type ( RCC_CoachStdMsg::CM_FREE ) 
  {}

  RCC_CoachAudio ( const int& t,
		   const RCC_Side& s, 
		   const int &ts,
		   RCC_Pointer < RCC_CoachInfoMsg > m ) 
    : 
    RCC_Audio ( t ),
    side_ ( s ),
    time_sent ( ts ),
    msg_p ( ( RCC_CoachStdMsg* ) m ),
    info_p ( m ), 
    msg_type ( RCC_CoachStdMsg::CM_INFO ) 
  {}

  RCC_CoachAudio ( const int& t,
		   const RCC_Side& s, 
		   const int &ts,
		   RCC_Pointer < RCC_CoachAdviceMsg > m ) 
    : 
    RCC_Audio ( t ),
    side_ ( s ),
    time_sent ( ts ),
    msg_p ( ( RCC_CoachStdMsg* ) m ),
    adv_p ( m ), 
    msg_type ( RCC_CoachStdMsg::CM_ADV ) 
  {}

  RCC_CoachAudio ( const int& t,
		   const RCC_Side& s, 
		   const int &ts,
		   RCC_Pointer < RCC_CoachDefineMsg > m ) 
    : RCC_Audio ( t ),
      side_ ( s ),
      time_sent ( ts ),
      msg_p ( ( RCC_CoachStdMsg* ) m ),
      def_p ( m ), 
      msg_type ( RCC_CoachStdMsg::CM_DEF ) 
  {}

  virtual ~RCC_CoachAudio ();

  RCC_Pointer < RCC_Object > copy () const 
  {
    switch ( msg_type )
      {
      case RCC_CoachStdMsg::CM_META:
	{
	  RCC_Pointer < RCC_CoachMetaMsg > msg_copy;
	  msg_copy.staticCast ( msg_p->copy () );
	  return new RCC_CoachAudio ( time (), side_, time_sent, msg_copy );
	}
      case RCC_CoachStdMsg::CM_FREE:
	{
	  RCC_Pointer < RCC_CoachFreeformMsg > msg_copy;
	  msg_copy.staticCast ( msg_p->copy () );
	  return new RCC_CoachAudio ( time (), side_, time_sent, msg_copy );
	}
      case RCC_CoachStdMsg::CM_INFO:
	{
	  RCC_Pointer < RCC_CoachInfoMsg > msg_copy;
	  msg_copy.staticCast ( msg_p->copy () );
	  return new RCC_CoachAudio ( time (), side_, time_sent, msg_copy );
	}
      case RCC_CoachStdMsg::CM_ADV:
	{	
	  RCC_Pointer < RCC_CoachAdviceMsg > msg_copy;
	  msg_copy.staticCast ( msg_p->copy () );
	  return new RCC_CoachAudio ( time (), side_, time_sent, msg_copy );
	}
      case RCC_CoachStdMsg::CM_DEF:
	{
	  RCC_Pointer < RCC_CoachDefineMsg > msg_copy;
	  msg_copy.staticCast ( msg_p->copy () );
	  return new RCC_CoachAudio ( time (), side_, time_sent, msg_copy );
	}
      default:
	return NULL;
      };
  }

  virtual std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const; 


  RCC_Side side () const { return side_; }
  RCC_Side side ( const RCC_Side& s ) { return side_ = s; }

  int timeSent () const { return time_sent; }
  int timeSent ( const int& ts ) { return time_sent = ts; }
  
  RCC_Pointer < RCC_CoachMetaMsg >
  metaMsg () 
  {
    if ( msg_type == RCC_CoachStdMsg::CM_META )
      return meta_p; 
    return NULL;
  }
 
  RCC_Pointer < RCC_CoachMetaMsg >
  setMsg ( RCC_Pointer < RCC_CoachMetaMsg > m ) 
  {
    msg_type = RCC_CoachStdMsg::CM_META;
    msg_p = ( RCC_CoachStdMsg* ) m;
    return meta_p = m;
  }

  RCC_Pointer < RCC_CoachFreeformMsg >
  freeMsg () 
  {
    if ( msg_type == RCC_CoachStdMsg::CM_FREE )
      return free_p; 
    return NULL;
  }
 
  RCC_Pointer < RCC_CoachFreeformMsg >
  setMsg ( RCC_Pointer < RCC_CoachFreeformMsg > m ) 
  {
    msg_type = RCC_CoachStdMsg::CM_FREE;
    msg_p = ( RCC_CoachStdMsg* ) m;
    return free_p = m;
  }

  RCC_Pointer < RCC_CoachInfoMsg >
  infoMsg () 
  {
    if ( msg_type == RCC_CoachStdMsg::CM_INFO )
      return info_p; 
    return NULL;
  }
 
  RCC_Pointer < RCC_CoachInfoMsg >
  setMsg ( RCC_Pointer < RCC_CoachInfoMsg > m ) 
  {
    msg_type = RCC_CoachStdMsg::CM_INFO;
    msg_p = ( RCC_CoachStdMsg* ) m;
    return info_p = m;
  }

  RCC_Pointer < RCC_CoachAdviceMsg >
  advMsg () 
  {
    if ( msg_type == RCC_CoachStdMsg::CM_ADV )
      return adv_p; 
    return NULL;
  }
 
  RCC_Pointer < RCC_CoachAdviceMsg >
  setMsg ( RCC_Pointer < RCC_CoachAdviceMsg > m ) 
  {
    msg_type = RCC_CoachStdMsg::CM_ADV;
    msg_p = ( RCC_CoachStdMsg* ) m;
    return adv_p = m;
  }

  RCC_Pointer < RCC_CoachDefineMsg >
  defMsg () 
  {
    if ( msg_type == RCC_CoachStdMsg::CM_DEF )
      return def_p; 
    return NULL;
  }
 
  RCC_Pointer < RCC_CoachDefineMsg >
  setMsg ( RCC_Pointer < RCC_CoachDefineMsg > m ) 
  {
    msg_type = RCC_CoachStdMsg::CM_DEF;
    msg_p = ( RCC_CoachStdMsg* ) m;
    return def_p = m;
  }

  RCC_CoachStdMsg::coach_msg_t
  type () const
  { return msg_type; }
  
private:   
  RCC_Side side_;
  int time_sent;

  RCC_Pointer < RCC_CoachStdMsg > msg_p; // used to keep the ref count above 1;
  union
  {
    RCC_CoachMetaMsg*		meta_p;
    RCC_CoachFreeformMsg*	free_p;
    RCC_CoachInfoMsg*		info_p;
    RCC_CoachAdviceMsg*		adv_p;
    RCC_CoachDefineMsg*		def_p;
  };
  RCC_CoachStdMsg::coach_msg_t	msg_type;

};

#endif
