// -*-c++-*-

/***************************************************************************
                          rcccoachtoken.h  -  Interface of RoboCup coach
			                      language token classes
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _COACH_TOKEN_H_
#define _COACH_TOKEN_H_

#pragma interface

#include <iostream>
#include <vector>

#include "rccregion.h"
#include "param.h"
#include "rccaction.h"
#include "rcccond.h"
#include "rccdirective.h"
#include "rccobject.h"
#include "rccmemory.h"
#include "rccstring.h"

class RCC_CoachToken : public RCC_Object
{
public:
  RCC_CoachToken() {}
  virtual ~RCC_CoachToken() {}

  virtual std::ostream& 
  print ( std::ostream& out ) const = 0;

  virtual std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header) const = 0;

  friend std::ostream& 
  operator << ( std::ostream& os, 
		const RCC_CoachToken& t )
  { return t.print(os); } 
};


class RCC_CoachMetaToken 
  : public RCC_CoachToken
{
public:
  typedef RCC_Container < RCC_CoachMetaToken > container;

  RCC_CoachMetaToken() {}
  virtual ~RCC_CoachMetaToken() {}
};

class RCC_CoachMetaToken_Version : public RCC_CoachMetaToken 
{
public:
  typedef RCC_Container < RCC_CoachMetaToken_Version > container;

  RCC_CoachMetaToken_Version ( double v = 0.0 ) 
    : ver_ ( v ) 
  {}
  ~RCC_CoachMetaToken_Version() {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_CoachMetaToken_Version ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "(ver " << ver_ << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const 
  { return out << line_header << "version: " << ver_ << std::endl; }

  double 
  ver ( const double& v ) 
  { return ver_ = v; }
  
  double ver () const 
  { return ver_; }
      
private:
  double ver_;
};


class RCC_CoachInfoToken : public RCC_CoachToken
{
public:
  typedef RCC_Container < RCC_CoachInfoToken > container;

  RCC_CoachInfoToken() {}
  virtual ~RCC_CoachInfoToken() {}
};


class RCC_CoachInfoToken_Clear : public RCC_CoachInfoToken
{
public:
  RCC_CoachInfoToken_Clear () {}
  ~RCC_CoachInfoToken_Clear () {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_CoachInfoToken_Clear (); }

  std::ostream& 	
  print ( std::ostream& out ) const 
  { return out << "(clear)"; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "CLEAR token" << std::endl; }
};


class RCC_CoachInfoToken_Rule 
  : public RCC_CoachInfoToken
{
public:
  RCC_CoachInfoToken_Rule ( const int& ttl, 
			    const RCC_Pointer < RCC_CondUnion >& c, 
			    const RCC_Pointer < RCC_DirUnion::container >& d );

  ~RCC_CoachInfoToken_Rule () {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  {
    RCC_Pointer < RCC_CondUnion > cond_copy;
    RCC_Pointer < RCC_DirUnion::container > dirs_copy;
    cond_copy.staticCast ( cond_p->copy () );
    dirs_copy.staticCast ( dirs_p->copy () );
    return new RCC_CoachInfoToken_Rule ( time_to_live, cond_copy, dirs_copy ); 
  } 

  std::ostream& 
  print ( std::ostream& out ) const;

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  /* class specific stuff */
  int 
  timeToLive () const 
  { return time_to_live; }

  int 
  timeToLive ( const int& ttl ) 
  { return time_to_live = ttl; }

  const RCC_Pointer < RCC_CondUnion > 
  cond () const 
  { return cond_p; }

  RCC_Pointer < RCC_CondUnion > 
  cond () 
  { return cond_p; }
  
  void 
  condition ( const RCC_Pointer < RCC_CondUnion >& c ) 
  { cond_p = c; }

  const RCC_Pointer < RCC_DirUnion::container > 
  dirs () const 
  { return dirs_p; }
  
  RCC_Pointer < RCC_DirUnion::container > 
  dirs () 
  { return dirs_p; }
  
  RCC_Pointer < RCC_DirUnion::container > 
  dirs ( const RCC_Pointer < RCC_DirUnion::container >& d )
  { return dirs_p = d; }

private:
  int time_to_live;
  RCC_Pointer < RCC_CondUnion > cond_p;
  RCC_Pointer < RCC_DirUnion::container > dirs_p;
};



class RCC_CoachDefineToken 
  : public RCC_CoachToken 
{
public:
  typedef RCC_Container < RCC_CoachDefineToken > container;

  RCC_CoachDefineToken ( const RCC_String& n ) : name_ ( n ) {}
  virtual ~RCC_CoachDefineToken() {}

  RCC_String name () const { return name_; }
  void  name ( const RCC_String& n ) { name_ = n; }

private:
  RCC_String name_;
};

class RCC_CoachDefineToken_Condition 
  : public RCC_CoachDefineToken
{
public:
  typedef RCC_Container < RCC_CoachDefineToken_Condition > container;

  RCC_CoachDefineToken_Condition ( const RCC_String& n, 
				   const RCC_Pointer < RCC_CondUnion >& c ) 
    :
    RCC_CoachDefineToken ( n ), 
    cond_p ( c ) 
  {}

  ~RCC_CoachDefineToken_Condition () {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  {
    RCC_Pointer < RCC_CondUnion > cond_copy;
    cond_copy.staticCast ( cond_p->copy () );
    return new RCC_CoachDefineToken_Condition ( name (), cond_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { 
    return out << "(definec \"" << name () 
	       << "\" " << *( cond_p->cond () ) << ")"; 
  }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
    { 
      out << line_header << "Condition \"" << name () << "\"" << std::endl;
      return cond_p->cond ()->printPretty ( out, line_header + " ");  
    }

private:
  RCC_Pointer < RCC_CondUnion > cond_p;
};


class RCC_CoachDefineToken_Directive 
  : public RCC_CoachDefineToken
{
public:
  typedef RCC_Container < RCC_CoachDefineToken_Directive > container;

  RCC_CoachDefineToken_Directive ( const RCC_String& n, 
				   const RCC_Pointer < RCC_DirUnion >& d )
    : 
    RCC_CoachDefineToken( n ), 
    dir_p ( d ) 
  {}
  
  ~RCC_CoachDefineToken_Directive() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_DirUnion > dir_copy;
    dir_copy.staticCast ( dir_p->copy () );
    return new RCC_CoachDefineToken_Directive ( name (), dir_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "(defined \"" << name () << "\" " << *dir_p->dir () << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header) const
  { 
    out << line_header << "Directive \"" << name () << "\"" << std::endl;
    dir_p->dir ()->printPretty ( out, line_header + " "); 
    return out; 
  }

private:
  RCC_Pointer < RCC_DirUnion > dir_p;
};



class RCC_CoachDefineToken_Region : public RCC_CoachDefineToken
{
public:
  typedef RCC_Container < RCC_CoachDefineToken_Region > container;

  RCC_CoachDefineToken_Region ( const RCC_String& n,
				const RCC_Pointer < RCC_RegUnion >& r )
    : 
    RCC_CoachDefineToken ( n ), 
    reg_p ( r ) 
  {}
  
  ~RCC_CoachDefineToken_Region() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( reg_p->copy () );
    return new RCC_CoachDefineToken_Region ( name (), reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "(definer \"" << name () << "\" " 
	       << *( reg_p->reg () ) << ")"; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { out << line_header << "Region \"" << name () << "\"" << std::endl;
  reg_p->reg ()->printPretty ( out, line_header + " " ); return out; }

private:
  RCC_Pointer < RCC_RegUnion > reg_p;
};

class RCC_CoachDefineToken_Action : public RCC_CoachDefineToken
{
public:
  typedef RCC_Container < RCC_CoachDefineToken_Action > container;

  RCC_CoachDefineToken_Action ( const RCC_String& n, 
				const RCC_Pointer < RCC_ActUnion >& a )
    : 
    RCC_CoachDefineToken ( n ), 
    act_p ( a ) 
  {}
  
  ~RCC_CoachDefineToken_Action() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_ActUnion > act_copy;
    act_copy.staticCast ( act_p->copy () );
    return new RCC_CoachDefineToken_Action ( name (), act_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return act_p->act ()->print ( out << "(definea \"" << name () << "\" " )
				      << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, const RCC_String& line_header ) const
  { out << line_header << "Action \"" << name () << "\"" << std::endl;
  act_p->act ()->printPretty ( out, line_header + " "); return out; }

private:
  RCC_Pointer < RCC_ActUnion > act_p;
};

#endif
