// -*-c++-*-

/***************************************************************************
                          rccdirective.h  -  Interface for RoboCup coach
			                     language directive clases
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_DIRECTIVE_H_
#define _RCC_DIRECTIVE_H_

#pragma interface

#include <iostream>
#include <vector>


#include "rccaction.h"
#include "rccside.h"
#include "rccobject.h"
#include "rccmemory.h"

class RCC_Directive : public RCC_Object
{
public:
  enum dir_type_t
  {
    DIR_VOID,
    DIR_COM,
    DIR_NAMED
  };

  typedef RCC_Container < RCC_Directive > container;

  RCC_Directive() {}
  virtual ~RCC_Directive() {}
  
  virtual std::ostream& 
  print ( std::ostream& out ) const = 0;

  virtual std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const = 0;

  friend std::ostream& 
  operator << ( std::ostream & os, const RCC_Directive& d )
  { return d.print(os); } 
};

class RCC_DirCommand;
class RCC_DirNamed;

class RCC_DirUnion
  : public RCC_Object
{
public:
  typedef RCC_Container < RCC_DirUnion > container;

  RCC_DirUnion ( RCC_Pointer < RCC_DirCommand > dir )
    : dir_p ( dir ),
      dir_com_p ( dir ),
      type_ ( RCC_Directive::DIR_COM )
  {}

  RCC_DirUnion ( RCC_Pointer < RCC_DirNamed > dir )
    : dir_p ( dir ),
      dir_named_p ( dir ),
      type_ ( RCC_Directive::DIR_NAMED )
  {}

  ~RCC_DirUnion () {}

  RCC_Pointer < RCC_Object >
  copy () const
  {
    switch ( type_ )
      {
      case RCC_Directive::DIR_COM:
	{
	  RCC_Pointer < RCC_DirCommand > dir_copy
	    = dir_p->copy ().staticCast < RCC_DirCommand > ();
	  return new RCC_DirUnion ( dir_copy );
	}

      case RCC_Directive::DIR_NAMED:
	{
	  RCC_Pointer < RCC_DirNamed > dir_copy
	    = dir_p->copy ().staticCast < RCC_DirNamed > ();
	  return new RCC_DirUnion ( dir_copy );
	}

      default:
	return NULL;
      };
  }

  RCC_Directive::dir_type_t 
  type () const
  { return type_; }

  RCC_Pointer < RCC_DirCommand > 
  setDir ( RCC_Pointer < RCC_DirCommand > dir )
  {
    dir_p = dir;
    dir_com_p = dir;
    type_ = RCC_Directive::DIR_COM;
    return dir;
  }

  RCC_Pointer < RCC_DirNamed > 
  setDir ( RCC_Pointer < RCC_DirNamed > dir )
  {
    dir_p = dir;
    dir_named_p = dir;
    type_ = RCC_Directive::DIR_NAMED;
    return dir;
  }

  RCC_Pointer < RCC_DirCommand >
  dirCom ()
  {
    if ( type_ == RCC_Directive::DIR_COM )
      return dir_com_p;
    return NULL;
  }

  RCC_Pointer < RCC_DirNamed >
  dirNamed ()
  {
    if ( type_ == RCC_Directive::DIR_NAMED )
      return dir_named_p;
    return NULL;
  }

  RCC_Pointer < RCC_Directive >
  dir ()
  {
    return dir_p;
  }

  const RCC_Pointer < RCC_DirCommand >
  dirCom () const
  {
    if ( type_ == RCC_Directive::DIR_COM )
      return dir_com_p;
    return NULL;
  }

  const RCC_Pointer < RCC_DirNamed >
  dirNamed () const
  {
    if ( type_ == RCC_Directive::DIR_NAMED )
      return dir_named_p;
    return NULL;
  }

  const RCC_Pointer < RCC_Directive >
  dir () const
  {
    return dir_p;
  }


private:
  RCC_Pointer < RCC_Directive > dir_p;  //used to keep ref count above zero
  union
  {
    RCC_DirCommand*	dir_com_p;
    RCC_DirNamed*	dir_named_p;
  };

  RCC_Directive::dir_type_t type_;
};

class RCC_DirCommand 
  : public RCC_Directive
{
public:
  RCC_DirCommand ( const bool& pos, 
		   const RCC_Side& s,
		   const RCC_UnumSet& p, 
		   const RCC_Pointer < RCC_ActUnion >& a )
    : 
    positive_ ( pos ), 
    side_ ( s ), 
    players ( p ), 
    act_p ( a ) {}

  ~RCC_DirCommand() {}

  RCC_Pointer < RCC_Object >
  copy () const 
  {
    RCC_Pointer < RCC_ActUnion > act_copy;
    act_copy.staticCast ( act_p->copy () );
    return new RCC_DirCommand ( positive_, side_, players, act_copy ); 
  }

  std::ostream&
  print ( std::ostream& out ) const;
 
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const; 

  bool positive () const { return positive_; }
  bool negative () const { return !positive_; }
  bool ourSide() const { return side_.ourSide (); }
  bool oppSide() const { return side_.oppSide (); }
  const RCC_UnumSet& playerSet() const { return players; }

  RCC_Pointer < RCC_ActUnion > 
  action() 
  { return act_p; }
  
  const RCC_Pointer < RCC_ActUnion > 
  action() const 
  { return act_p; }
  
  RCC_Pointer < RCC_ActUnion > 
  action ( RCC_Pointer < RCC_ActUnion > act ) 
  { return act_p = act; }

  void positive ( const bool& pos ) { positive_ = pos; }
  void side ( const RCC_Side& s ) { side_ = s; }
  void playerSet (const RCC_UnumSet& p ) { players = p; }
  void addPlayer ( const unsigned int& i) { players.add ( i ); }
  void removePlayer ( const unsigned int& i) { players.remove ( i ); }
  void clearPlayers () { players.clear(); }
  
private:
  bool positive_; //true = do; false = dont
  RCC_Side side_; //true = our; false = opp
  RCC_UnumSet players;
  RCC_Pointer < RCC_ActUnion > act_p;
} ;


class RCC_DirNamed 
  : public RCC_Directive
{
public:
  RCC_DirNamed ( const RCC_String& n ) : name_ ( n ) {}
  ~RCC_DirNamed() {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_DirNamed ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const 
  { out << "\"" << name_ << "\""; return out; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "directived named \"" 
	       << name_ << "\"" << std::endl; }

  /* class specific */
  RCC_String name () const { return name_; }
  void  name ( const RCC_String& n ) { name_ = n; } 

private:
  RCC_String name_;
};

#endif
