// -*-c++-*-

/***************************************************************************
                          rccinitmsg.h  -  Interface and implementation of
			                   RoboCup initialisation message
					   class.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_INIT_MSG_H_
#define _RCC_INIT_MSG_H_

#include "rccplaymode.h"
#include <iostream>
#include "types.h"
#include "rccsensordata.h"

class RCC_InitMsg : public RCC_SensorData
{
public:
  RCC_InitMsg ( const Side& s, const int& un, const RCC_StadPlayMode& pm ) 
    : side_ ( s ), unum ( un ), play_mode ( pm ) {}
  RCC_InitMsg ( const Side& s, const RCC_StadPlayMode& pm ) 
    : side_ ( s ), unum ( -1 ), play_mode ( pm ) {}

  ~RCC_InitMsg () {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_InitMsg ( *this ); }

  friend std::ostream& 
  operator<< ( std::ostream& o, 
	       const RCC_InitMsg& m )
  {
    return m.print ( o );
  }
  
  std::ostream& 
  print ( std::ostream& o ) const 
  {
    if ( reconnect() )
      o << "(reconnect " << ( side_ == LEFT ? "l" : "r" );
    else
      o << "(init " << ( side_ == LEFT ? "l" : "r" ) << " "<< unum << " ";
		
    return play_mode.print ( o, side_ ) << ")";
  }

  std::ostream& 
  print ( std::ostream& o, 
	  const Side& ) const 
  { return print ( o ); } 
  
  Side 
  side () const 
  { return side_; }
  
  Side 
  side ( const Side& s )
  { return side_ = s; }
  
  int 
  uNum () const
  { return unum; }
  
  int 
  uNum ( const int& un )
  { return unum = un; }
  
  RCC_StadPlayMode 
  playMode () const
  { return play_mode; }

  RCC_StadPlayMode 
  playMode ( const RCC_StadPlayMode& pm )
  { return play_mode = pm; }

  bool
  reconnect () const
  { return unum == -1; }


protected:
  Side			side_;
  int			unum;
  RCC_StadPlayMode 	play_mode;
};

#endif
