// -*-c++-*-

/***************************************************************************
                          rcclockable.cc  -  Interface and implementation of
			                     a generic intface to allow mutal
					     exclusion.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_LOCKABLE_H_
#define _RCC_LOCKABLE_H_

#include "config.h"
#ifdef HAVE_PTHREAD_H		
#include <pthread.h>
#endif //HAVE_PTHREAD_H		

class RCC_Lockable
{
public:
#ifdef HAVE_PTHREAD_H		
  RCC_Lockable ( const pthread_mutexattr_t* attr = NULL ) 
#else //HAVE_PTHREAD_H
  RCC_Lockable ( const void* = NULL ) 
#endif //HAVE_PTHREAD_H		
  {
#ifdef HAVE_PTHREAD_H		
    pthread_mutex_init ( &mutex_, attr );
#endif //HAVE_PTHREAD_H		
  }

  virtual ~RCC_Lockable () 
  {
#ifdef HAVE_PTHREAD_H		
    pthread_mutex_destroy ( &mutex_ );
#endif //HAVE_PTHREAD_H		

  }

  void lock () const 
  { 
#ifdef HAVE_PTHREAD_H		
    pthread_mutex_lock ( &mutex_ ); 
#endif //HAVE_PTHREAD_H		
  } 
  
  void unlock () const 
  { 
#ifdef HAVE_PTHREAD_H		
    pthread_mutex_unlock ( &mutex_ ); 
#endif //HAVE_PTHREAD_H		
  } 
  
  void tryLock () const 
  { 
#ifdef HAVE_PTHREAD_H		
    pthread_mutex_trylock ( &mutex_ ); 
#endif //HAVE_PTHREAD_H		
  }
 
#ifdef HAVE_PTHREAD_H		
  pthread_mutex_t& mutex ()
  { return this->mutex_; }
#endif //HAVE_PTHREAD_H		

private:
#ifdef HAVE_PTHREAD_H		
  mutable pthread_mutex_t mutex_;
#endif //HAVE_PTHREAD_H		
};

#endif //_RCC_LOCKABLE_H_
