// -*-c++-*-

/***************************************************************************
                          rccparser.h  -  Interface for the RoboCup language
			                  parsing class.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_PARSER_H_
#define _RCC_PARSER_H_

#pragma interface

#include "types.h"

#include "rccsensordata.h"
#include "rccsubstitution.h"
#include "rccscore.h"
#include "rccerror.h"
#include "rccaudio.h"
#include "rccinitmsg.h"
#include "rccsensebody.h"
#include "rccvisual.h"
#include "rccserverparam.h"
#include "rccplayerparam.h"
#include "rccplayertype.h"
#include "rccstring.h"
#include "config.h"
#include <list>
#include "rccmemory.h"
#include "rccexception.h"
#include "rcctransmitter.h"
#include "rccstopwatch.h"
#include "rccfullstate.h"

class RCC_ParserTmpStore
{
public:
  RCC_ParserTmpStore () {}
  ~RCC_ParserTmpStore () {}

  template < class T > 
  T* 
  add ( T* ptr ) 
  { 
    list_RCC_Object.push_back ( ptr ); 
    return ptr; 
  }
private:
  std::list < RCC_Pointer < RCC_Object > > 	list_RCC_Object;
};




//=============================================================================
// A class for passing parameters to RCC_MainParse
//=============================================================================

class RCC_MP_Param
{
public:
  
  enum msg_type_t
  {
    VOID_MSG,
    FULL_STATE_MSG,
    SENSE_BODY_MSG,
    SERVER_PARAM_MSG,
    PLAYER_PARAM_MSG,
    PLAYER_TYPE_MSG,
    SEE_MSG,
    SCORE_MSG,
    INIT_MSG,
    HEAR_MSG,
    CHANGE_MSG,
    ERROR_MSG,
  };

  RCC_MP_Param ( const RCC_MP_Param::msg_type_t& t = RCC_MP_Param::VOID_MSG )
    : type_ ( t )
  {}

  ~RCC_MP_Param ()
  {}

  RCC_MP_Param::msg_type_t
  type () const
  { return type_; }

  RCC_MP_Param::msg_type_t
  type ( const RCC_MP_Param::msg_type_t& t )
  { return type_ = t; }

private:
  RCC_MP_Param::msg_type_t	type_;
};


int RCC_MainParse ( const RCC_String&, RCC_MP_Param& );

//=============================================================================
// A class for passing parameters to RCC_SBParse
//=============================================================================

class RCC_SBP_Param
{
public:
  RCC_SBP_Param ( const int& m,
		  RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& trans,
		  RCC_Pointer < RCC_SensorData >& d_p )
    : mode_ ( m ),
      transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

  int
  mode () const
  { return mode_; }

  int
  mode ( const int& m )
  { return mode_ = m; }

private:
  int mode_;
  RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore	ts;
};

int RCC_SBParse ( const RCC_String&, RCC_SBP_Param& );

//=============================================================================
// A class for passing parameters to RCC_FSParse
//=============================================================================

class RCC_FSP_Param
{
public:
  RCC_FSP_Param ( const int& m, 
		  const Side& s,
		  RCC_Transmitter < RCC_Pointer < RCC_FullState > >& trans,
		  RCC_Pointer < RCC_SensorData >& d_p )
    : mode_ ( m ),
      side_ ( s ),
      transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_FullState > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_FullState > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_FullState > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_FullState > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

  int
  mode () const
  { return mode_; }

  int
  mode ( const int& m )
  { return mode_ = m; }

  Side
  side () const
  { return side_; }

  Side
  side ( const Side& s )
  { return side_ = s; }

private:
  int mode_;
  Side side_;
  RCC_Transmitter < RCC_Pointer < RCC_FullState > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};

int RCC_FSParse ( const RCC_String&, RCC_FSP_Param& );

//=============================================================================
// A class for passing parameters to RCC_SPParse
//=============================================================================

class RCC_SPP_Param
{
public:
  RCC_SPP_Param ( RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& trans,
		  RCC_Pointer < RCC_SensorData >& d_p )
    : transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

private:
  RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};


int RCC_SPParse ( const RCC_String&, RCC_SPP_Param& );


//=============================================================================
// A class for passing parameters to RCC_PPParse
//=============================================================================

class RCC_PPP_Param
{
public:
  RCC_PPP_Param ( RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& trans,
		  RCC_Pointer < RCC_SensorData >& d_p )
    : transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

private:
  RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};


int RCC_PPParse ( const RCC_String&, RCC_PPP_Param& );


//=============================================================================
// A class for passing parameters to RCC_PTParse
//=============================================================================

class RCC_PTP_Param
{
public:
  RCC_PTP_Param ( RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& trans,
		  RCC_Pointer < RCC_SensorData >& d_p )
    : transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

private:
  RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};


int RCC_PTParse ( const RCC_String&, RCC_PTP_Param& );


//=============================================================================
// A class for passing parameters to RCC_VisParse
//=============================================================================

class RCC_VP_Param
{
public:
  RCC_VP_Param ( const int& m, 
		 const RCC_String& name,
		 RCC_Transmitter < RCC_Pointer < RCC_Visual > >& trans,
		 RCC_Pointer < RCC_SensorData >& d_p )
    : mode_ ( m ),
      tn ( name ),
      transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_Visual > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_Visual > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_Visual > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_Visual > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

  int
  mode () const
  { return mode_; }

  int
  mode ( const int& m )
  { return mode_ = m; }

  RCC_String
  teamName () const
  { return tn; }

  RCC_String
  teamName ( const RCC_String& name )
  { return tn = name; }

private:
  int mode_;
  RCC_String tn;
  RCC_Transmitter < RCC_Pointer < RCC_Visual > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};

int RCC_VisParse ( const RCC_String&, RCC_VP_Param& );


//=============================================================================
// A class for passing parameters to RCC_AudParse
//=============================================================================

class RCC_AP_Param
{
public:
  RCC_AP_Param ( const Side& s, 
		 RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& r_trans,
		 RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& s_trans,
		 RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& p_trans,
		 RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& c_trans,
		 RCC_Pointer < RCC_SensorData >& d_p )
    : side_ ( s ),
      ref_trans ( r_trans ),
      self_trans ( s_trans ),
      player_trans ( p_trans ),
      coach_trans ( c_trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& 
  refTrans ()
  { return ref_trans; }

  const RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& 
  refTrans () const
  { return ref_trans; }

  RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& 
  refTrans ( const RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& trans )
  { return ref_trans = trans; }

  RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& 
  selfTrans ()
  { return self_trans; }

  const RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& 
  selfTrans () const
  { return self_trans; }

  RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& 
  Trans ( const RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& trans )
  { return self_trans = trans; }

  RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& 
  playerTrans ()
  { return player_trans; }

  const RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& 
  playerTrans () const
  { return player_trans; }

  RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& 
  playerTrans ( const RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& trans )
  { return player_trans = trans; }

  RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& 
  coachTrans ()
  { return coach_trans; }

  const RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& 
  coachTrans () const
  { return coach_trans; }

  RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& 
  coachTrans ( const RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& trans )
  { return coach_trans = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

  Side
  side () const
  { return side_; }

  Side
  side ( const Side& s )
  { return side_ = s; }

private:
  Side side_;
  RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& ref_trans;
  RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& self_trans;
  RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& player_trans;
  RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& coach_trans;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};

int RCC_AudParse ( const RCC_String&, RCC_AP_Param& );


//=============================================================================
// A class for passing parameters to RCC_ScParse
//=============================================================================

class RCC_SCP_Param
{
public:
  RCC_SCP_Param ( RCC_Transmitter < RCC_Pointer < RCC_Score > >& trans,
		  RCC_Pointer < RCC_SensorData >& d_p )
    : transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_Score > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_Score > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_Score > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_Score > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

private:
  RCC_Transmitter < RCC_Pointer < RCC_Score > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};

int RCC_ScParse ( const RCC_String&, RCC_SCP_Param& );


//=============================================================================
// A class for passing parameters to RCC_InitParse
//=============================================================================

class RCC_IP_Param
{
public:
  RCC_IP_Param ( Side& s,
		 RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& trans,
		 RCC_Pointer < RCC_SensorData >& d_p )
    : side_ ( s ),
      transmitter_ ( trans ),
      data_p ( d_p )
  {}

  RCC_Pointer < RCC_SensorData >&
  data ()
  { return data_p; }

  const RCC_Pointer < RCC_SensorData >&
  data () const
  { return data_p; }

  RCC_Pointer < RCC_SensorData >&
  data ( const RCC_Pointer < RCC_SensorData >& d_p )
  { return data_p = d_p; }

  RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& 
  transmitter ()
  { return transmitter_; }

  const RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& 
  transmitter () const
  { return transmitter_; }

  RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& 
  transmitter ( const RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& trans )
  { return transmitter_ = trans; }

  RCC_ParserTmpStore&
  tmpStore ()
  { return ts; }

  Side&
  side ()
  { return side_; }

  const Side&
  side () const
  { return side_; }

  Side
  side ( Side& s )
  { return side_ = s; }

private:
  Side& side_;
  RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& transmitter_;
  RCC_Pointer < RCC_SensorData >& data_p; 
  RCC_ParserTmpStore ts;
};

int RCC_InitParse ( const RCC_String&, RCC_IP_Param& );


//=============================================================================
// A class for passing parameters to RCC_Parse
//=============================================================================


class RCC_Parser;

struct rccparam_t
{
  // input
  const int&		mode;
  const RCC_String& 	tn;
  const RCC_Parser&	transmitter;
  // input and output
  Side&   	side;
  
  // output
  RCC_Pointer < RCC_SensorData >& 	sd_p;

  // local
  RCC_ParserTmpStore	ts;

  rccparam_t ( const int&			m,			
	       const RCC_String& 		team_name,
	       const RCC_Parser&		trans,
	       Side&		   		team_side,
	       RCC_Pointer < RCC_SensorData >&	sensor_data,
	       RCC_ParserTmpStore		tmp_store = RCC_ParserTmpStore()
	       )
    : mode ( m ),
      tn ( team_name ),
      transmitter ( trans ),
      side ( team_side ),
      sd_p ( sensor_data ),
      ts ( tmp_store )
  {}
};

int RCC_Parse ( const RCC_String& s, 
		  const int& mode,
		  const RCC_String& team_name,
		  const RCC_Parser& transmitter,
		  Side& side, 
		  RCC_Pointer < RCC_SensorData >& data_p
		  );


class RCC_Parser
  : public RCC_Handler < RCC_Pointer < RCC_String > >
{
public:
  enum mode_t { FIELD_PLAYER, ONLINE_COACH, OFFLINE_COACH };

#ifdef DEBUG
  static int compare ( const RCC_String& in, const RCC_String& out );
#endif //DEBUG


  RCC_Parser ( const RCC_Parser::mode_t& m = RCC_Parser::FIELD_PLAYER,
	       const std::string& tn = "",
	       const Side& s = NEUTRAL ) 
    : mode_ ( m ), team_name ( tn ), side_ ( s ) 
  {
    if ( mode_ != FIELD_PLAYER )
      throw RCC_Exception ( "non-field player parseing not implemented... yet" );
  }

  virtual ~RCC_Parser ( ) { }
  
  RCC_Parser::mode_t mode ( const RCC_Parser::mode_t& m ) { return mode_ = m; }
  RCC_String teamName ( const RCC_String& tn ) { return team_name = tn; }
  Side side ( const Side& s ) { return side_ = s; }

  RCC_Pointer < RCC_SensorData > 
  parse ( const RCC_String& str )
  {
    RCC_Pointer < RCC_SensorData > data_p = NULL;
#ifdef DEBUG
    static int count = 0;
    static RCC_StopWatch timer;

    try
      {
	timer.unPause ();
      }
    catch (...)
      {
	timer.start ();
      }
#endif //DEBUG
    if ( str != "" )
      {
	RCC_MP_Param main_param;
	RCC_MainParse ( str, main_param );
	switch ( main_param.type () )
	  {
	  case RCC_MP_Param::FULL_STATE_MSG:
	    {
	      RCC_FSP_Param param ( mode_, side_, fsTrans (), data_p );
	      RCC_FSParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::SENSE_BODY_MSG:
	    {
	      RCC_SBP_Param param ( mode_, senseBodyTrans (), data_p );
	      RCC_SBParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::SERVER_PARAM_MSG:
	    {
	      RCC_SPP_Param param ( serverParamTrans (), data_p );
	      RCC_SPParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::PLAYER_PARAM_MSG:
	    {
	      RCC_PPP_Param param ( playerParamTrans (), data_p );
	      RCC_PPParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::PLAYER_TYPE_MSG:
	    {
	      RCC_PTP_Param param ( playerTypeTrans (), data_p );
	      RCC_PTParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::SEE_MSG:
	    {
	      RCC_VP_Param param ( mode_, team_name, visualTrans (), data_p );
	      RCC_VisParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::HEAR_MSG:
	    {
	      RCC_AP_Param param ( side_, 
				   refAudioTrans (),
				   selfAudioTrans (),
				   playerAudioTrans (),
				   coachAudioTrans (),
				   data_p );
	      RCC_AudParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::SCORE_MSG:
	    {
	      RCC_SCP_Param param ( scoreTrans (), data_p );
	      RCC_ScParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::INIT_MSG:
	    {
	      RCC_IP_Param param ( side_, initMsgTrans (), data_p );
	      RCC_InitParse ( str, param );
	      break;
	    }
	  case RCC_MP_Param::CHANGE_MSG:
	  case RCC_MP_Param::ERROR_MSG:
	    {
	      RCC_Parse ( str, mode_, team_name, *this, side_, data_p );
	      break;
	    }
	  default:
	    std::cerr << "unknown message type\n";
	    break;
	  }	
	if ( data_p != NULL )
	  {
#ifdef DEBUG
	    ++count;
	    char out[MaxMesg];
	    std::strstream stream ( out, MaxMesg );

	    data_p->print ( stream, side_ ) << std::ends;
	
	    if ( RCC_Parser::compare ( str, out ) == 0 )
	      {
		timer.pause ();
		return data_p;
	      }
	    else
	      {
		std::cerr << " input>" << str << "<\n";
		std::cerr << "output>" << out << "<\n";
		return NULL;
	      }
#else //!DEBUG
	    return data_p;
#endif //DEBUG
	  }
	else // RCC_Parse ( str, mode_, team_name, side_, data_p ) != 0
	  {
	    std::cerr << "error parseing input>" << str << "<\n";
	    return NULL;
	  }
      } 
    else
      {
#ifdef DEBUG
	timer.stop ();
	std::cout << "total (p) = " << timer.time () << std::endl;
	std::cout << "ave (p) = " << timer.time () / count << std::endl;
#endif //DEBUG
	return NULL;
      }
  }

  

  virtual void handle ( const RCC_Pointer < RCC_String >& str_p )
  {
    parse ( *str_p );
  }

  void transmit ( const RCC_Pointer < RCC_SenseBody >& item ) const
  { senseBodyTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_ServerParam >& item ) const
  { serverParamTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_PlayerParam >& item ) const
  { playerParamTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_PlayerType >& item ) const
  { playerTypeTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_Visual >& item ) const
  { visualTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_InitMsg >& item ) const
  { initMsgTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_RefAudio >& item ) const
  { refAudioTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_SelfAudio >& item ) const
  { selfAudioTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_PlayerAudio >& item ) const
  { playerAudioTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_CoachAudio >& item ) const
  { coachAudioTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_Score >& item ) const
  { scoreTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_Substitution >& item ) const
  { substitutionTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_Error >& item ) const
  { errorTrans ().transmit ( item ); }
  void transmit ( const RCC_Pointer < RCC_FullState >& item ) const
  { fsTrans ().transmit ( item ); }

  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_SenseBody > >& handler )
  { senseBodyTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_ServerParam > >& handler )
  { serverParamTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_PlayerParam > >& handler )
  { playerParamTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_PlayerType > >& handler )
  { playerTypeTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_Visual > >& handler )
  { visualTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_InitMsg > >& handler )
  { initMsgTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_RefAudio > >& handler )
  { refAudioTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_SelfAudio > >& handler )
  { selfAudioTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_PlayerAudio > >& handler )
  { playerAudioTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_CoachAudio > >& handler )
  { coachAudioTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_Score > >& handler )
  { scoreTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_Substitution > >& handler )
  { substitutionTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_Error > >& handler )
  { errorTrans ().registerHandler ( handler ); }
  void 
  registerHandler ( RCC_Handler < RCC_Pointer < RCC_FullState > >& handler )
  { fsTrans ().registerHandler ( handler ); }
  
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_SenseBody > >& handler )
  { senseBodyTrans ().deRegisterHandler ( handler ); }
  void
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_ServerParam > >& handler )
  { serverParamTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_PlayerParam > >& handler )
  { playerParamTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_PlayerType > >& handler )
  { playerTypeTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_Visual > >& handler )
  { visualTrans ().deRegisterHandler ( handler );  }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_InitMsg > >& handler )
  { initMsgTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_RefAudio > >& handler )
  { refAudioTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_SelfAudio > >& handler )
  { selfAudioTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_PlayerAudio > >& handler )
  { playerAudioTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_CoachAudio > >& handler )
  { coachAudioTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_Score > >& handler )
  { scoreTrans ().deRegisterHandler ( handler ); }
  void 
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_Substitution > >& handler )
  { substitutionTrans ().deRegisterHandler ( handler ); }
  void
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_Error > >& handler )
  { errorTrans ().deRegisterHandler ( handler ); }
  void
  deRegisterHandler ( RCC_Handler < RCC_Pointer < RCC_FullState > >& handler )
  { fsTrans ().deRegisterHandler ( handler ); }
 
  RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >& 	senseBodyTrans ()
  { return sb_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >& 	serverParamTrans ()
  { return sp_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >& 	playerParamTrans ()
  { return pp_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >& 	playerTypeTrans ()
  { return pt_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_Visual > >& 	visualTrans () 
  { return v_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >& 	initMsgTrans ()
  { return im_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >& 	refAudioTrans ()
  { return ra_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >& 	selfAudioTrans ()
  { return sa_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >& 	playerAudioTrans ()
  { return pa_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >& 	coachAudioTrans ()
  { return ca_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_Score > >& 	scoreTrans ()
  { return score_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_Substitution > >& substitutionTrans ()
  { return sub_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_Error > >& 	errorTrans ()
  { return e_transmitter; }
  RCC_Transmitter < RCC_Pointer < RCC_FullState > >& 	fsTrans ()
  { return fs_transmitter; }

  const RCC_Transmitter < RCC_Pointer < RCC_SenseBody > >&
  senseBodyTrans () const
  { return sb_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_ServerParam > >&
  serverParamTrans () const
  { return sp_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > >&
  playerParamTrans () const 	
  { return pp_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_PlayerType > >&
  playerTypeTrans () const
  { return pt_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_Visual > >&
  visualTrans () const
  { return v_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_InitMsg > >&
  initMsgTrans () const
  { return im_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_RefAudio > >&
  refAudioTrans () const
  { return ra_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > >&
  selfAudioTrans () const
  { return sa_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > >&
  playerAudioTrans () const
  { return pa_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > >&
  coachAudioTrans () const
  { return ca_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_Score > >&
  scoreTrans () const
  { return score_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_Substitution > >&
  substitutionTrans () const
  { return sub_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_Error > >&
  errorTrans () const
  { return e_transmitter; }
  const RCC_Transmitter < RCC_Pointer < RCC_FullState > >&
  fsTrans () const
  { return fs_transmitter; }

private:
  RCC_Parser::mode_t mode_;
  RCC_String team_name;
  Side side_;

  RCC_Transmitter < RCC_Pointer < RCC_SenseBody > > sb_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_ServerParam > > sp_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_PlayerParam > > pp_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_PlayerType > > pt_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_Visual > > v_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_InitMsg > > im_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_RefAudio > > ra_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_SelfAudio > > sa_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_PlayerAudio > > pa_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_CoachAudio > > ca_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_Score > > score_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_Substitution > > sub_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_Error > > e_transmitter;
  RCC_Transmitter < RCC_Pointer < RCC_FullState > > fs_transmitter;
};	

#ifdef HAVE_PTHREAD_H
#include "rccthread.h"
#include "rcctransqueue.h"

class RCC_ParserThread
  : public RCC_Thread,
    public RCC_Parser
{
public:
  RCC_ParserThread ( const RCC_Parser::mode_t& m = RCC_Parser::FIELD_PLAYER,
		     const std::string& tn = "",
		     const Side& s = NEUTRAL ) 
    : RCC_Parser ( m, tn, s ) 
  {
  }

  void start ()
  {
    _start ( this );
  }

  void* main ()
  {
    for (;;)
      {
	RCC_Pointer < RCC_String > str_p = trans.frontAndPop ();
	if ( parse ( *str_p ) == NULL )
	  return NULL;
      }
  }

  void handle ( const RCC_Pointer < RCC_String >& str_p )
  {
    trans.handle ( str_p );
  }


private:
  RCC_TransQueue < RCC_Pointer < RCC_String > > trans;
};

#endif //HAVE_PTHREAD_H

#endif
