// -*-c++-*-

/***************************************************************************
                          rccplayertype.h  -  Interface for RoboCup player type
			                      messages.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_PLAYER_TYPE_H_
#define _RCC_PLAYER_TYPE_H_

#pragma interface

#include <iostream>
#include "rccsensordata.h"

class RCC_PlayerType : public RCC_SensorData
{
public:
  static const int UNKNOWN_ID;

  RCC_PlayerType ( const int& id,
		   const double& player_speed_max,
		   const double& stamina_inc_max,
		   const double& player_decay,
		   const double& inertia_moment,
		   const double& dash_power_rate,
		   const double& player_size,
		   const double& kickable_margin,
		   const double& kick_rand,
		   const double& extra_stamina,
		   const double& effort_max,
		   const double& effort_min );

  ~RCC_PlayerType () {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_PlayerType ( *this ); }
   
  friend std::ostream& operator<< (std::ostream& o, const RCC_PlayerType& sp)
  {
    return sp.print ( o );
  }

  std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 

  int typeId () const { return id; }
  double playerSpeedMax () const { return player_speed_max; }
  double staminaIncMax () const { return stamina_inc_max; }
  double playerDecay () const { return player_decay; }
  double inertiaMoment () const { return inertia_moment; }
  double dashPowerRate () const { return dash_power_rate; }
  double playerSize () const { return player_size; }
  double kickableMargin () const { return kickable_margin; }
  double kickRand () const { return kick_rand; }
  double extraStamina() const { return extra_stamina; }
  double effortMax () const { return effort_max; }
  double effortMin () const { return effort_min; }

private:
  int id;
  double player_speed_max;
  double stamina_inc_max;
  double player_decay;
  double inertia_moment;
  double dash_power_rate;
  double player_size;
  double kickable_margin;
  double kick_rand;
  double extra_stamina;
  double effort_max;
  double effort_min;
};


#endif
