// -*-c++-*-

/***************************************************************************
                          rccplaymode.h  -  Interface for RoboCup play mode
			                    classes
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_PLAY_MODE_H_
#define _RCC_PLAY_MODE_H_

#pragma interface

#include "param.h"
#include "types.h"
#include "rccside.h"
#include "rccobject.h"
#include "rccmemory.h"

#include "rccstring.h"
#include <iostream>


class RCC_PlayMode : public RCC_Object
{
public:
  RCC_PlayMode () {}
  virtual ~RCC_PlayMode () {}

  friend std::ostream& operator<< ( std::ostream& o, const RCC_PlayMode& pm )
  { return pm.print ( o ); }

  virtual std::ostream& print ( std::ostream& o ) const = 0;
};


class RCC_SidedPlayMode
{
public:
  RCC_SidedPlayMode ( const RCC_Side& s = RCC_Side::Neut )
    : side_ ( s ) {}
  ~RCC_SidedPlayMode () {}

  RCC_Side side () const { return side_; }
  RCC_Side side ( const RCC_Side& s ) { return side_ = s; }

private:
  RCC_Side	side_;
};




class RCC_NumericPlayMode
{
public:
  RCC_NumericPlayMode ( const int& v = -1 )
    : val ( v ) {}

  ~RCC_NumericPlayMode ()
  {}

  int value () const { return val; }
  int value ( const int& v ) { return val = v; }

private:
  int val;
};





class RCC_StadPlayMode 
  : public RCC_PlayMode, 
    public RCC_SidedPlayMode,
    public RCC_NumericPlayMode
{
public:
  enum mode_t {
    Unknown,
    BeforeKickOff,
    TimeOver,
    PlayOn,
    KickOff_,
    KickIn_,
    FreeKick_,
    CornerKick_,
    GoalKick_,
    AfterGoal_,
    Drop_Ball,
    OffSide_,
    PK_,
    FirstHalfOver,
    Pause,
    Human,
    Foul_Charge_,
    Foul_Push_,
    Foul_MultipleAttacker_,
    Foul_BallOut_,
    GoalieCatch_,
    TimeUpWithoutTeam,
    TimeUp,
    TimeExtended,
    HalfTime,
    Foul_,
    Goal_
  };

  static const RCC_String STAD_NAMES[];

  RCC_StadPlayMode ( const RCC_StadPlayMode::mode_t& m, 
		     const RCC_Side& s = RCC_Side::Neut )
    : RCC_SidedPlayMode ( s ), 
      mode_ ( m )
  {
  }

  RCC_StadPlayMode ( const RCC_Side& s, const int& points )
    : RCC_SidedPlayMode ( s ), 
    RCC_NumericPlayMode ( points ),
      mode_ ( RCC_StadPlayMode::Goal_ )
  {
  }

  ~RCC_StadPlayMode () {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_StadPlayMode ( *this ); }

  std::ostream& print ( std::ostream& o ) const
  { return print ( o, NEUTRAL ); }

  std::ostream& print ( std::ostream& o, const Side& s ) const
  {
    o << RCC_StadPlayMode::STAD_NAMES [ mode_ ];
    if ( !side ().neutSide () )
      {
	if ( s == NEUTRAL )
	  o << side ();
	else
	  side ().print ( o, s );
      }
    if ( mode () == RCC_StadPlayMode::Goal_ )
      o << "_" << value ();
    return o;
  }

  RCC_StadPlayMode::mode_t mode () const { return mode_; }
  RCC_StadPlayMode::mode_t mode ( const RCC_StadPlayMode::mode_t& m )
  { return mode_ = m; }

private:
  RCC_StadPlayMode::mode_t mode_;
};


class RCC_CoachPlayMode
  : public RCC_PlayMode,
    public RCC_SidedPlayMode
{
public:
  enum coach_mode_t {
    none,
    bko,
    time_over,
    play_on,
    ko_,		
    ki_,		
    fk_,
    ck_,
    gk_,
    gc_,
    ag_
  };
  
  static const RCC_String COACH_NAMES[];

  RCC_CoachPlayMode ( const RCC_CoachPlayMode::coach_mode_t& m, const RCC_Side& s = RCC_Side::Neut )
    : RCC_SidedPlayMode ( s ), mode_ ( m )
  {}
 
  RCC_CoachPlayMode ( const RCC_StadPlayMode::mode_t& m )
  {
    switch ( m )
      {
      case RCC_StadPlayMode::TimeOver:
	mode_ = RCC_CoachPlayMode::time_over;
	break;

      case RCC_StadPlayMode::PlayOn:
	mode_ = RCC_CoachPlayMode::play_on;
	break;

      default:
	throw RCC_Exception ( "Invalid playmode in initialisation of RCC_CoachPlayMode" );
      }
  }

  ~RCC_CoachPlayMode () {}

  RCC_Pointer < RCC_Object > copy () const
  { return new RCC_CoachPlayMode ( *this ); }

  RCC_CoachPlayMode::coach_mode_t mode () const { return mode_; }
  RCC_CoachPlayMode::coach_mode_t mode ( const RCC_CoachPlayMode::coach_mode_t& m )
  { return mode_ = m; }

  std::ostream& print ( std::ostream& o ) const
  {
    o << RCC_CoachPlayMode::COACH_NAMES [ mode_ ];
    if ( !side ().neutSide () )
      o << side ();
    return o;
  }

private:
  RCC_CoachPlayMode::coach_mode_t mode_;
};


#endif
