// -*-c++-*-

/***************************************************************************
                          rccsensebody.h  -  Interface for RoboCup sense body
			                     messages.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_SENSEBODY_H_
#define _RCC_SENSEBODY_H_

#pragma interface

#include <iostream>
#include "rccviewmode.h"
#include "rccangle.h"
#include "rccvector2d.h"
#include "rccsensordata.h"
#include "rccmemory.h"

class RCC_SenseBody : public RCC_SensorData
{
public:

  RCC_SenseBody ( const int& time, const RCC_ViewMode& vm, const double& stam, const double &eff, 
		  const RCC_Vector2d& v, const RCC_Angle& ha,
		  const int& kick_c, 
		  const int& dash_c, const int& turn_c, 
		  const int& say_c, const int& turn_neck_c, const int& catch_c, const int& move_c, const int& chg_view_c);
  ~RCC_SenseBody ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_SenseBody ( *this ); }

  int time () const { return bi_time; }
  int time ( const int& t ) { return ( bi_time = t ); }

  RCC_ViewMode viewMode () const { return view_mode; }
  RCC_ViewMode viewMode ( const RCC_ViewMode& vm ) { return view_mode = vm; }

  double stamina () const { return stamina_; }
  double stamina ( const double& s ) { return ( stamina_ = s ); }

  double effort () const { return effort_; }
  double effort ( const double& e ) { return ( effort_ = e ); }

  RCC_Vector2d vel () const { return velocity; }
  RCC_Vector2d vel ( const RCC_Vector2d &v ) { return ( velocity = v ); }

  RCC_Angle headAngle () const { return head_angle; }
  RCC_Angle headAngle ( const RCC_Angle& ha ) { return ( head_angle = ha ); }

  int kickCount () const { return kick_count; }
  int kickCount ( const int & c ) { return ( kick_count = c ); }
    
  int dashCount () const { return dash_count; }
  int dashCount ( const int & c ) { return ( dash_count = c ); }

  int turnCount () const { return turn_count; }
  int turnCount ( const int & c ) { return ( turn_count = c ); }
    
  int sayCount () const { return say_count; }
  int sayCount ( const int & c ) { return ( say_count = c ); }
    
  int turnNeckCount () const { return turn_neck_count; }
  int turnNeckCount ( const int & c ) { return ( turn_neck_count = c ); }

  int catchCount () const { return catch_count; }
  int catchCount ( const int & c ) { return ( catch_count = c ); }

  int moveCount () const { return move_count; }
  int moveCount ( const int & c ) { return ( move_count = c ); }

  int changeViewCount () const { return change_view_count; }
  int changeViewCount ( const int & c ) { return ( change_view_count = c ); }

  std::ostream& 
  print ( std::ostream& o, 
	  const Side& ) const 
  { return print ( o ); } 
  
  std::ostream& 
  print ( std::ostream& o ) const; 
   
  friend std::ostream& 
  operator<< ( std::ostream& o, 
	       const RCC_SenseBody& bi) 
  { return bi.print( o ); }

private:
  int bi_time;

  RCC_ViewMode view_mode;
    
  double stamina_;
  double effort_;
  RCC_Vector2d velocity;
  RCC_Angle head_angle;

  int kick_count;
  int dash_count;
  int turn_count;
  int say_count;
  int turn_neck_count;
  int catch_count;
  int move_count;
  int change_view_count;
};

#endif 
