// -*-c++-*-

/***************************************************************************
                          rccregion.h  -  Interface for RoboCup server 
			                  parameter message class
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_SERVER_PARAM_H_
#define _RCC_SERVER_PARAM_H_

#pragma interface

#include "rccangle.h"
#include "rccsensordata.h"

class RCC_ServerParam : public RCC_SensorData
{
public:
  RCC_ServerParam ( const double& par_gwidth, 
		    const double& par_inertia_moment, 
		    const double& par_psize, 
		    const double& par_pdecay, 
		    const double& par_prand,
		    const double& par_pweight, 
		    const double& par_pspeed_max, 
		    const double& par_paccel_max, 
		    const double& par_stamina_max, 
		    const double& par_stamina_inc,
		    const double& par_recover_init, 
		    const double& par_recover_dthr, 
		    const double& par_recover_min, 
		    const double& par_recover_dec, 
		    const double& par_effort_init,
		    const double& par_effort_dthr, 
		    const double& par_effort_min, 
		    const double& par_effort_dec, 
		    const double& par_effort_ithr, 
		    const double& par_effort_inc,
		    const double& par_kick_rand, 
		    const int& par_team_actuator_noise, 
		    const double& par_prand_factor_l, 
		    const double& par_prand_factor_r, 
		    const double& par_kick_rand_factor_l,
		    const double& par_kick_rand_factor_r, 
		    const double& par_bsize, 
		    const double& par_bdecay, 
		    const double& par_brand, 
		    const double& par_bweight,
		    const double& par_bspeed_max, 
		    const double& par_baccel_max, 
		    const double& par_dprate, 
		    const double& par_kprate, 
		    const double& par_kmargin,
		    const double& par_ctlradius, 
		    const double& par_ctlradius_width, 
		    const double& par_maxp, 
		    const double& par_minp, 
		    const RCC_Angle& par_maxm,
		    const RCC_Angle& par_minm, 
		    const RCC_Angle& par_maxnm, 
		    const RCC_Angle& par_minnm, 
		    const RCC_Angle& par_maxn, 
		    const RCC_Angle& par_minn,
		    const RCC_Angle& par_visangle, 
		    const double& par_visdist, 
		    const RCC_Angle& par_windir, 
		    const double& par_winforce, 
		    const RCC_Angle& par_winang,
		    const double& par_winrand, 
		    const double& par_kickable_area, 
		    const double& par_catch_area_l, 
		    const double& par_catch_area_w, 
		    const double& par_catch_prob,
		    const int& par_goalie_max_moves, 
		    const double& par_ckmargin, 
		    const double& par_offside_area, 
		    const int& par_win_no, 
		    const int& par_win_random,
		    const int& par_say_cnt_max, 
		    const int& par_SayCoachMsgSize, 
		    const int& par_clang_win_size, 
		    const int& par_clang_define_win, 
		    const int& par_clang_meta_win,
		    const int& par_clang_advice_win, 
		    const int& par_clang_info_win, 
		    const int& par_clang_mess_delay, 
		    const int& par_clang_mess_per_cycle, 
		    const int& par_half_time,
		    const int& par_sim_st, 
		    const int& par_send_st, 
		    const int& par_recv_st, 
		    const int& par_sb_step, 
		    const int& par_lcm_st,
		    const int& par_SayMsgSize, 
		    const int& par_hear_max, 
		    const int& par_hear_inc, 
		    const int& par_hear_decay, 
		    const int& par_cban_cycle,
		    const int& par_slow_down_factor, 
		    const int& par_useoffside, 
		    const int& par_kickoffoffside, 
		    const double& par_offside_kick_margin, 
		    const double& par_audio_dist,
		    const double& par_dist_qstep, 
		    const double& par_land_qstep, 
		    const double& par_dir_qstep, 
		    const double& par_dist_qstep_l, 
		    const double& par_dist_qstep_r,
		    const double& par_land_qstep_l, 
		    const double& par_land_qstep_r, 
		    const double& par_dir_qstep_l, 
		    const double& par_dir_qstep_r, 
		    const int& par_CoachMode,
		    const int& par_CwRMode, 
		    const int& par_old_hear, 
		    const int& par_sv_st, 
		    const int& par_start_goal_l, 
		    const int& par_start_goal_r,
		    const int& par_fullstate_l, 
		    const int& par_fullstate_r, 
		    const int& par_drop_time);

  ~RCC_ServerParam () {}
   
  friend std::ostream& operator<< (std::ostream& o, const RCC_ServerParam& sp)
  {
    return sp.print ( o );
  }

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ServerParam ( *this ); }

  std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 

  double goalWidth () const { return gwidth; } 					
  double inertiaMoment () const { return inertia_moment; } 	
  double playerSize () const { return psize; } 			
  double playerDecay () const { return pdecay; } 			
  double playerRand () const { return prand; } 
			
  double playerWeight () const { return pweight; } 				
  double playerSpeedMax () const { return pspeed_max; } 		
  double playerAccelMax () const { return paccel_max; } 		
  double staminaMax () const { return stamina_max; } 			
  double staminaInc () const { return stamina_inc; } 
			
  double recoveryInit () const { return recover_init; } 		
  double recoveryDecThr () const { return recover_dthr; } 		
  double recoveryMin () const { return recover_min; } 			
  double recoveryDec () const { return recover_dec; } 			
  double effortInit () const { return effort_init; } 
			
  double effortDecThr () const { return effort_dthr; } 			
  double effortMin () const { return effort_min; } 		
  double effortDec () const { return effort_dec; } 			
  double effortIncThr () const { return effort_ithr; } 			
  double effortInc () const { return effort_inc; }
 			
  double kickRand () const { return kick_rand; }                             
  int teamActNoise () const { return team_actuator_noise; }                     
  double playerRandFactorLeft () const { return prand_factor_l; }                          
  double playerRandFactorRight () const { return prand_factor_r; }                            
  double kickRandFactorLeft () const { return kick_rand_factor_l; }
                       
  double kickRandFactorRight () const { return kick_rand_factor_r; }                     
  double BallSize () const { return bsize; } 			
  double BallDecay () const { return bdecay; } 			
  double BallRand () const { return brand; } 			
  double BallWeight () const { return bweight; }
 				
  double BallSpeedMax () const { return bspeed_max; } 			
  double BallAccelMax () const { return baccel_max; } 			
  double DashPowerRate () const { return dprate; } 			
  double KickPowerRate () const { return kprate; } 			
  double KickMargin () const { return kmargin; }
 				
  double ControlRadius () const { return ctlradius; } 			
  double ControlRadiusWidth () const { return ctlradius_width; } 		
  double maxPower () const { return maxp; }				
  double minPower () const { return minp; } 				
  RCC_Angle maxMoment () const { return maxm; }
 				
  RCC_Angle minMoment () const { return minm; } 				
  RCC_Angle maxNeckMoment () const { return maxnm; } 				
  RCC_Angle minNeckMoment () const { return minnm; } 				
  RCC_Angle maxNeckAngle () const { return maxn; } 				
  RCC_Angle minNeckAngle () const { return minn; }
 				
  RCC_Angle visibleAngle () const { return visangle; } 			
  double visibleDist () const { return visdist; } 					
  RCC_Angle windDir () const { return windir; } 					
  double windForce () const { return winforce; } 				
  RCC_Angle windAngle () const { return winang; }
 					
  double windRand () const { return winrand; } 					
  double kickArea () const { return kickable_area; } 			
  double catchAreaLength () const { return catch_area_l; } 			
  double catchAreaWidth () const { return catch_area_w; } 			
  double catchProb() const { return catch_prob; } 
				
  int goalieMaxMoves () const { return goalie_max_moves; }                  	
  double cornerKickMargin () const { return ckmargin; } 				
  double offsideArea () const { return offside_area; } 			
  int windNone () const { return win_no; } 					
  int windRandom () const { return win_random; } 
				
  int sayCountMax () const { return say_cnt_max; } 				
  int sayCoachMsgSize () const { return SayCoachMsgSize; } 				
  int coachLangWindowSize () const { return clang_win_size; }
  int coachLangDefineWindow () const { return clang_define_win; }
  int coachLangMetaWindow () const { return clang_meta_win; }

  int coachLangAdviceWindow () const { return clang_advice_win; }
  int coachLangInfoWindow () const { return clang_info_win; }
  int coachLangMsgDelay () const { return clang_mess_delay; }
  int coachMsgPerCycle () const { return clang_mess_per_cycle; }
  int halfTime () const { return half_time; }
 					
  int simStep () const { return sim_st; } 					
  int sendStep () const { return send_st; } 					
  int recvStep () const { return recv_st; } 					
  int senceBodyStep () const { return sb_step; } 					
  int LCMStep () const { return lcm_st; }
 		                        
  int sayMsgSize () const { return SayMsgSize; } 				
  int hearMax () const { return hear_max; } 					
  int hearInc () const { return hear_inc; } 					
  int hearDecay () const { return hear_decay; } 				
  int catchBanCycle () const { return cban_cycle; }
 				
  int slowDownFactor () const { return slow_down_factor; }                           
  int useOffside () const { return useoffside; } 				
  int kickoffOffside() const { return kickoffoffside; } 			
  double offsideKickMargin () const { return offside_kick_margin; } 		
  double audioDist () const { return audio_dist; } 
				
  double distQuantizeStep () const { return dist_qstep; } 				
  double landQuantizeStep () const { return land_qstep; } 				
  double dirQuantizeStep () const { return dir_qstep; } 				
  double distQuantizeStepLeft () const { return dist_qstep_l; } 			
  double distQuantizeStepRight () const { return dist_qstep_r; } 
			
  double landQuantizeStepLeft () const { return land_qstep_l; } 			
  double landQuantizeStepRight () const { return land_qstep_r; } 			
  double dirQuantizeStepLeft () const { return dir_qstep_l; } 				
  double dirQuantizeStepRight () const { return dir_qstep_r; } 				
  int coachMode () const { return CoachMode; }
 				
  int coachWithRefMode () const { return CwRMode; } 					
  int oldHear () const { return old_hear; } 					
  int coachSendVisualStep () const { return sv_st; }
  int startGoalLeft () const { return start_goal_l; }
  int startGoalRight () const { return start_goal_r; }

  int fullStateLeft () const { return fullstate_l; }
  int fullStateRight () const { return fullstate_r; }
  int dropTime () const { return drop_time; }

private:
  double gwidth; 					
  double inertia_moment; 	
  double psize; 			
  double pdecay; 			
  double prand; 
			
  double pweight; 				
  double pspeed_max; 		
  double paccel_max; 		
  double stamina_max; 			
  double stamina_inc; 
			
  double recover_init; 		
  double recover_dthr; 		
  double recover_min; 			
  double recover_dec; 			
  double effort_init; 
			
  double effort_dthr; 			
  double effort_min; 		
  double effort_dec; 			
  double effort_ithr; 			
  double effort_inc;
 			
  double kick_rand;                             
  int team_actuator_noise;                     
  double prand_factor_l;                          
  double prand_factor_r;                            
  double kick_rand_factor_l;
                       
  double kick_rand_factor_r;                     
  double bsize; 			
  double bdecay; 			
  double brand; 			
  double bweight;
 				
  double bspeed_max; 			
  double baccel_max; 			
  double dprate; 			
  double kprate; 			
  double kmargin;
 				
  double ctlradius; 			
  double ctlradius_width; 		
  double maxp;				
  double minp; 				
  RCC_Angle maxm;
 				
  RCC_Angle minm; 				
  RCC_Angle maxnm; 				
  RCC_Angle minnm; 				
  RCC_Angle maxn; 				
  RCC_Angle minn;
 				
  RCC_Angle visangle; 			
  double visdist; 					
  RCC_Angle windir; 					
  double winforce; 				
  RCC_Angle winang;
 					
  double winrand; 					
  double kickable_area; 			
  double catch_area_l; 			
  double catch_area_w; 			
  double catch_prob; 
				
  int goalie_max_moves;                  	
  double ckmargin; 				
  double offside_area; 			
  int win_no; 					
  int win_random; 
				
  int say_cnt_max; 				
  int SayCoachMsgSize; 				
  int clang_win_size;
  int clang_define_win;
  int clang_meta_win;

  int clang_advice_win;
  int clang_info_win;
  int clang_mess_delay;
  int clang_mess_per_cycle;
  int half_time;
 					
  int sim_st; 					
  int send_st; 					
  int recv_st; 					
  int sb_step; 					
  int lcm_st;
 		                        
  int SayMsgSize; 				
  int hear_max; 					
  int hear_inc; 					
  int hear_decay; 				
  int cban_cycle;
 				
  int slow_down_factor;                           
  int useoffside; 				
  int kickoffoffside; 			
  double offside_kick_margin; 		
  double audio_dist; 
				
  double dist_qstep; 				
  double land_qstep; 				
  double dir_qstep; 				
  double dist_qstep_l; 			
  double dist_qstep_r; 
			
  double land_qstep_l; 			
  double land_qstep_r; 			
  double dir_qstep_l; 				
  double dir_qstep_r; 				
  int CoachMode;
 				
  int CwRMode; 					
  int old_hear; 					
  int sv_st;
  int start_goal_l;
  int start_goal_r;

  int fullstate_l;
  int fullstate_r;
  int drop_time;
};


#endif
