// -*-c++-*-

/***************************************************************************
                          rccstopwatch.h  -  Interface and implementaion of
			                     a basic class for measuring time.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_STOP_WATCH_H_
#define _RCC_STOP_WATCH_H_

#include <sys/time.h>
#include <unistd.h>
#include "rccmemory.h"
#include "rccobject.h"
#include "rccexception.h"

inline timeval operator- ( const timeval& t1, const timeval& t2 )
{
  timeval tmp;
  tmp.tv_sec = t1.tv_sec - t2.tv_sec;
  tmp.tv_usec = t1.tv_usec - t2.tv_usec;
  if ( ( tmp.tv_sec > 0 && tmp.tv_usec  < 0 )  || tmp.tv_usec < -1000000 )
    {
      tmp.tv_sec--;
      tmp.tv_usec += 1000000;
    }
  else if ( ( tmp.tv_sec < 0 && tmp.tv_usec > 0 ) || tmp.tv_usec > 1000000 )
    {
      tmp.tv_sec++;
      tmp.tv_usec -= 1000000;
    }

  return tmp;
}

inline std::ostream& operator<< ( std::ostream& o, const timeval& tv )
{
  return o << tv.tv_sec << "s " << tv.tv_usec << "s";
}

inline timeval operator+( const timeval& t1, const timeval& t2 )
{
  timeval tmp;
  tmp.tv_sec = t1.tv_sec + t2.tv_sec;
  tmp.tv_usec = t1.tv_usec + t2.tv_usec;
  if ( ( tmp.tv_sec > 0 && tmp.tv_usec  < 0 )  || tmp.tv_usec < -1000000 )
    {
      tmp.tv_sec--;
      tmp.tv_usec += 1000000;
    }
  else if ( ( tmp.tv_sec < 0 && tmp.tv_usec > 0 ) || tmp.tv_usec > 1000000 )
    {
      tmp.tv_sec++;
      tmp.tv_usec -= 1000000;
    }

  return tmp;
}

inline timeval operator/( const timeval& t1, const double& q )
{
  double sec, usec;
  timeval tmp;
  sec = (t1.tv_sec / q);
  tmp.tv_sec = (long) sec;

  usec = (t1.tv_usec / q) + ( sec - tmp.tv_sec ) * 1000000;
  tmp.tv_usec = (long)usec;

  while ( tmp.tv_usec > 1000000 )
    {
      tmp.tv_sec++;
      tmp.tv_usec -= 1000000;      
    }
  while ( tmp.tv_usec < -1000000 )
    {
      tmp.tv_sec--;
      tmp.tv_usec += 1000000;      
    }
  return tmp;
}


class RCC_StopWatch : public RCC_Object
{
public:
  RCC_StopWatch () : running ( false ), paused ( false ) {}
  ~RCC_StopWatch () {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_StopWatch ( *this ); }
  
  timeval start () 
  {
    if ( !running )
      {
	running = true; 
	gettimeofday ( &start_time, NULL ); 
	return start_time;
      }
    else
      throw RCC_Exception ( "tried to start a running stopwatch" );
  }

  timeval stop () 
  { 
    if ( running ) 
      {
	running = false;
	gettimeofday ( &stop_time, NULL ); 
	return stop_time - start_time; 
      }
    else
      throw RCC_Exception ( "tried to stop a stopped stop watch" );
  }

  timeval pause ()
  {
    if ( running && !paused )
      {
	paused = true;
	return stop ();
      }
    else if ( !running )
      throw RCC_Exception ( "tried to pause a stopped stop watch" );
    else
      throw RCC_Exception ( "tried to pause a paused stop watch" );
  }

  timeval unPause ()
  {
    if ( !running && paused )
      {
	paused = false;
	timeval tmp = time ();
	timeval new_start = start ();
	return start_time = start_time - tmp;
      }
    else if ( running )
      throw RCC_Exception ( "tried to unpause a running stop watch" );
    else
      return start ();//throw RCC_Exception ( "tried to unpause a unpaused stop watch" );
  }

  timeval time () const
  {
    if ( running )
      {
	timeval tmp;
	gettimeofday ( &tmp, NULL );
	return tmp - start_time;
      }
    else
      return stop_time - start_time;
  }

  friend std::ostream& operator<< ( std::ostream& o, const RCC_StopWatch& sw )
  {
    return o << sw.time ();
  }

private:
  bool running;
  bool paused;
  timeval start_time;
  timeval stop_time;
};


#endif
