// -*-c++-*-

/***************************************************************************
                          rccstring.h  -  Interface and implementaion of a
			                  std::string that can be pointed to
					  with a RCC_Pointer.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_STRING_H_
#define _RCC_STRING_H_

#include "rccobject.h"
#include "rccmemory.h"
#include <string>
#include <strstream>

#define RCC_STRING_BUF_SIZE 128

class RCC_String : public std::string, public RCC_Object
{
public:
  RCC_String () {}
  RCC_String ( const RCC_String& str, size_type pos, size_type n = npos) : std::string ( str, pos, npos ) {}
  RCC_String ( const char* s, size_type n ) : std::string ( s, n ) {}
  RCC_String ( const char* str ) : std::string ( str ) {}
  RCC_String ( const std::string& str ) : std::string ( str ) {}
  RCC_String ( size_type n, char c ) : std::string ( n, c ) {}
  RCC_String ( const bool& b ) : std::string ( ( b ? "true" : "false" ) ) {}
  RCC_String ( const int& i )
  {
    char str [ RCC_STRING_BUF_SIZE ];
    std::strstream stream ( str, RCC_STRING_BUF_SIZE );
    stream << i << std::ends;
    *this = str;
  }
  RCC_String ( const unsigned int& i )
  {
    char str [ RCC_STRING_BUF_SIZE ];
    std::strstream stream ( str, RCC_STRING_BUF_SIZE );
    stream << i << std::ends;
    *this = str;
  }
  RCC_String ( const long& l )
  {
    char str [ RCC_STRING_BUF_SIZE ];
    std::strstream stream ( str, RCC_STRING_BUF_SIZE );
    stream << l << std::ends;
    *this = str;
  }
  RCC_String ( const unsigned long& l )
  {
    char str [ RCC_STRING_BUF_SIZE ];
    std::strstream stream ( str, RCC_STRING_BUF_SIZE );
    stream << l << std::ends;
    *this = str;
  }
  RCC_String ( const double& d, const int& prec = 6 )
  {
    char str [ RCC_STRING_BUF_SIZE ];
    std::strstream stream ( str, RCC_STRING_BUF_SIZE );
    stream.precision ( prec );
    stream << d << std::ends;
    *this = str;
  }
  ~RCC_String () {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_String ( *this ); }
};

#endif
