// -*-c++-*-

/***************************************************************************
                          rccsyncable.h  -  Interface and Implementaion of a
			                    basic class to allow 
					    synchronisation between threads.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_SYNCABLE_H_
#define _RCC_SYNCABLE_H_

#include "config.h"
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif // HAVE_PTHREAD_H

#include "rcclockable.h"

class RCC_Syncable 
  : public RCC_Lockable
{
public:
  RCC_Syncable ( pthread_condattr_t *cond_attr = NULL )
  {
#ifdef HAVE_PTHREAD_H
    int err = pthread_cond_init ( &cond, cond_attr );
#endif // HAVE_PTHREAD_H
  }

  ~RCC_Syncable ()
  {
#ifdef HAVE_PTHREAD_H
    int err = pthread_cond_destroy ( &cond );
#endif //HAVE_PTHREAD_H
  }

  void signal ()
  {
    this->lock ();
    this->preLockedSignal ();
    this->unlock ();
  }

  void preLockedSignal ()
  {
#ifdef HAVE_PTHREAD_H
    int err = pthread_cond_signal ( &cond );
#endif //HAVE_PTHREAD_H
  }

  void broadcast ()
  {
    this->lock ();
    this->preLockedBroadcast ();
    this->unlock ();
  }

  void preLockedBroadcast ()
  {
#ifdef HAVE_PTHREAD_H
    int err = pthread_cond_broadcast ( &cond );
#endif //HAVE_PTHREAD_H
  }

  void wait ()
  {
    this->lock ();
    this->preLockedWait ();
    this->unlock ();
  }  

  void preLockedWait ()
  {
#ifdef HAVE_PTHREAD_H
    int err = pthread_cond_wait ( &cond, &RCC_Lockable::mutex () );
#endif //HAVE_PTHREAD_H
  }  

private:
#ifdef HAVE_PTHREAD_H
  pthread_cond_t cond;
#endif // HAVE_PTHREAD_H

};

#endif
